# Generated by Django 4.2.5 on 2023-10-29 22:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("books", "0003_auto_20221110_0400"),
    ]

    operations = [
        migrations.AddField(
            model_name="book",
            name="type",
            field=models.IntegerField(choices=[(1, "Hardcover"), (2, "Softcover")], default=1),
        ),
    ]
