# Generated by Django 3.1.3 on 2020-12-05 14:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coffee', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='flavor',
            name='decimal_value',
            field=models.DecimalField(decimal_places=2, max_digits=10, null=True),
        ),
        migrations.AddField(
            model_name='flavor',
            name='float_value',
            field=models.FloatField(null=True),
        ),
    ]
