{% extends "www/base.html" %}
{% load static unicorn %}

{% block content %}

<h2>Index</h2>

<button onclick="Unicorn.call('unicorn.components.hello_world.HelloWorldView', 'set_name');">
  Method call from outside of the component
</button>

<button onclick="alert(Unicorn.getReturnValue('unicorn.components.hello_world.HelloWorldView'));">
  Last return value from component
</button>

<!-- abcdf should be ignored and name will be None -->
{% unicorn 'unicorn.components.hello_world.HelloWorldView' name=abcdf %}

{% endblock content %}
